# © 2016-2019 Resurface Labs Inc.

import json
import usagelogger
from usagelogger.usage_loggers import UsageLoggers
from urllib.parse import urlparse
import urllib.request


class BaseLogger(object):

    def __init__(self, agent, queue=None,
                 url=UsageLoggers.url_by_default(), enabled=True):

        self.agent = agent
        self.version = self._version_lookup()
        self.skip_compression = False
        self.skip_submission = False
        self._enabled = enabled
        self._url = None

        self.queue = queue
        # queue takes precedence over url
        if self.queue is not None:
            self.url = None
        elif url is not None:
            self.url = url
        else:
            self._enabled = False

    def disable(self):
        self._enabled = False

    def enable(self):
        if self.enableable:
            self._enabled = True

    @property
    def enableable(self):
        return self.queue is not None or self.url is not None

    @property
    def enabled(self):
        return self._enabled and UsageLoggers.is_enabled()

    def submit(self, submission):
        if submission is None or self.skip_submission is True or self.enabled is False:
            return True
        elif self.queue is not None:
            self.queue.append(submission)
            return True
        else:
            try:
                # TODO: replace this quick hack
                # TODO: implement compression
                # TODO: add specific exceptions for json parsing or http errors
                body = json.dumps(submission).encode('utf8')
                headers = {'content-type': 'application/json'}
                request = urllib.request.Request(self.url,
                                                 data=body,
                                                 headers=headers)
                response = urllib.request.urlopen(request)
                return response.getcode() == 204

            except Exception:
                return False

    @property
    def url(self):
        return self._url

    @url.setter
    def url(self, value):
        # Validate URL
        try:
            # urlparse will return a byte type for a scheme it doesn't
            # understand which will raise a TypeError when compared to a
            # string, so we will raise the same for no http and catch both
            if 'http' not in urlparse(value).scheme:
                raise TypeError('incorrect URL scheme')
            self._url = value
        except TypeError:
            self._url = None
            self._enabled = False

    @staticmethod
    def _version_lookup():
        return usagelogger.__version__



# © 2016-2019 Resurface Labs Inc.


class HttpRule(object):

    def __init__(self, verb, scope=None, param1=None, param2=None):
        self.verb = verb
        self.scope = scope
        self.param1 = param1
        self.param2 = param2



from setuptools import setup

setup(
    name='usagelogger',
    version='0.1.5',
    description='Logging usage of Python cloud apps',
    long_description='© 2016-2019 Resurface Labs Inc.\n\nLogging usage of Python cloud apps, with user privacy by design.',
    long_description_content_type='text/plain',
    url='https://github.com/resurfaceio/logger-python',
    author='Resurface Labs',
    author_email='admin@resurface.io',
    license="Apache License 2.0",
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: System Administrators',
        'Topic :: Internet :: Log Analysis',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7'
    ],
    keywords='logging resurface',
    packages=['usagelogger'],
    python_requires='>=3.4, <4',
    install_requires=[],
    tests_require=['expects', 'mamba'],
    project_urls={
        'Bug Reports': 'https://github.com/resurfaceio/logger-python/issues',
        'Source': 'https://github.com/resurfaceio/logger-python'
    }
)



# © 2016-2019 Resurface Labs Inc.


class HttpMessage(object):

    @staticmethod
    def build():
        message = []

        return message

    @staticmethod
    def _append_request_headers(message, request):
        pass

    @staticmethod
    def _append_request_params(message, request):
        pass

    @staticmethod
    def _append_response_headers(message, response):
        pass

    @staticmethod
    def _append_value(message, key, value=None):
        pass
